<?php


namespace OrganizationsManagement\Repository;


use Doctrine\ORM\EntityManager;
use OrganizationsManagement\Util\InstanceProvider;
use Teknisa\Model\Traits\TQuery;

class Notice
{
    use TQuery;
    /**
     * @var EntityManager
     */
    private $entityManager;

    /**
     * Notice constructor.
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function getNrorg() {
        $environment = InstanceProvider::getLibEnvironment();
        return $environment->getCurrentOrganizationId();
    }

    public function getCdoperador() {
        $environment = InstanceProvider::getLibEnvironment();
        return $environment->getCdOperador();
    }

    public function getNoticeRecords()
    {
        $query = <<<SQL
        SELECT INF.NRINFECOMMERCE,
               INF.DSTITULOINF,
               INF.TXCONTEUDOINF,
               INF.IDNOTIFICACAO,
               TO_CHAR(INF.DTINFORMARIVI, 'DD/MM/YYYY') AS DTINFORMARIVI,
               TO_CHAR(INF.DTEXPIRAINFO, 'DD/MM/YYYY') AS DTEXPIRAINFO,
               TO_CHAR(INF.DTINCLUSAO, 'DD/MM/YYYY') AS DTINCLUSAO,
               TO_CHAR(INF.DTULTATU, 'DD/MM/YYYY') AS DTULTATU,
               INF.IDATIVO,
               INF.CDPUBLICOALVO,
               INF.LINKDIRIMAGE,
               (
                    SELECT 
                        RTRIM(XMLAGG(XMLELEMENT(E, PROD.NRPRODUTO || ',')).EXTRACT('//text()'), ',') AS NRPRODUTOS
                    FROM 
                        INFORMATIVO_PRODUTO PROD
                    WHERE 
                        PROD.NRINFECOMMERCE = INF.NRINFECOMMERCE
                ) AS NRPRODUTOS
          FROM INFORMATIVOECOMMERCE INF
          WHERE INF.CDPUBLICOALVO != 'L'
          AND NRORG = :NRORG
SQL;
        $params = array(
            "NRORG" => $this->getNrorg()
        );

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }

    public function insertNotice($data)
    {
        $query = <<<SQL
        INSERT
          INTO INFORMATIVOECOMMERCE
               (NRINFECOMMERCE,
               DSTITULOINF,
               TXCONTEUDOINF,
               IDNOTIFICACAO,
               DTINFORMARIVI,
               DTEXPIRAINFO,
               DTINCLUSAO,
               DTULTATU,
               IDATIVO,
               CDPUBLICOALVO,
               NRORG,
               NRORGINCLUSAO,
               CDOPERINCLUSAO,
               LINKDIRIMAGE)
        VALUES (:ID,
               :TITULO,
               :CONTEUDO,
               :TIPO,
               TRUNC(CAST(:DATAINICIO AS DATE)),
               TRUNC(CAST(:DATATERMINO AS DATE)),
               TRUNC(CAST(SYSDATE AS DATE)),
               TRUNC(CAST(SYSDATE AS DATE)),
               :ATIVO,
               :CDPUBLICOALVO,
               :NRORG,
               :NRORGINCLUSAO,
               :CDOPERINCLUSAO,
               :LINKDIRIMAGE)
SQL;
            $params = array(
                'ID' => $data['id'],
                'TITULO' => $data['title'],
                'CONTEUDO' => $data['content'],
                'TIPO' => $data['type'],
                'DATAINICIO' => $data['dateBegin'],
                'DATATERMINO' => $data['dateEnd'],
                'ATIVO' => $data['active'],
                'CDPUBLICOALVO' => $data['publicoAlvoNotice'],
                'NRORG'         => $this->getNrorg(),
                'NRORGINCLUSAO' => $this->getNrorg(),
                'CDOPERINCLUSAO' => $this->getCdoperador(),
                'LINKDIRIMAGE' => $data['linkDir'],
            );

        $entityManager = InstanceProvider::getLibConnectionCustom()->updateVpdConnection($this->getNrorg());
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        return $entityManager->getConnection()->executeQuery($query, $params);   
    }

    public function deleteNotice($id)
    {
        $query = <<<SQL
        DELETE
            FROM INFORMATIVOECOMMERCE
            WHERE NRINFECOMMERCE = :NRINFECOMMERCE
            AND NRORG = :NRORG
SQL;
        $params = array (
            'NRORG' => $this->getNrorg(),
            'NRINFECOMMERCE'    => $id,
        );
        $entityManager = InstanceProvider::getLibConnectionCustom()->updateVpdConnection($this->getNrorg());
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        return $entityManager->getConnection()->executeQuery($query, $params);   
    }

    public function deleteEcomOper($id)
    {
        $query = <<<SQL
                DELETE
                  FROM INFORMATIVOECOMOPER
                 WHERE NRINFECOMMERCE = :NRINFECOMMERCE
                 AND NRORG = :NRORG
SQL;
        $params = array (
            'NRORG' => $this->getNrorg(),
            'NRINFECOMMERCE'    => $id,
        );
        $entityManager = InstanceProvider::getLibConnectionCustom()->updateVpdConnection($this->getNrorg());
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        return $entityManager->getConnection()->executeQuery($query, $params);
    }

    public function updateNotice($item)
    {
        $query = <<<SQL
        UPDATE INFORMATIVOECOMMERCE 
        SET DSTITULOINF = :TITULO, 
            TXCONTEUDOINF = :CONTEUDO, 
            IDNOTIFICACAO = :TIPO, 
            DTINFORMARIVI = TRUNC(CAST(:DATAINICIO AS DATE)),
            DTEXPIRAINFO = TRUNC(CAST(:DATATERMINO AS DATE)), 
            DTULTATU = TRUNC(CAST(SYSDATE AS DATE)),
            IDATIVO = :ATIVO,
            NRORG = :NRORG,
            NRORGULTATU = :NRORGULTATU,
            CDOPERULTATU  = :CDOPERULTATU,
            LINKDIRIMAGE = :LINKDIRIMAGE
        WHERE NRINFECOMMERCE = :ID
        AND NRORG = :NRORG
SQL;
        $params = array(
            'ID' => $item['id'],
            'TITULO' => $item['title'],
            'CONTEUDO' => $item['content'],
            'TIPO' => $item['type'],
            'DATAINICIO' => $item['dateBegin'],
            'DATATERMINO' => $item['dateEnd'],
            'ATIVO' => $item['active'],
            'CDPUBLICOALVO' => $item['publicoAlvoNoticeEdit'],
            'NRORG' => $this->getNrorg(),
            'NRORGULTATU' => $this->getNrorg(),
            'CDOPERULTATU'  => $this->getCdoperador(),
            'LINKDIRIMAGE' => $item['linkDir'],
        );
        $entityManager = InstanceProvider::getLibConnectionCustom()->updateVpdConnection($this->getNrorg());
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        return $entityManager->getConnection()->executeQuery($query, $params);
    }

    public function getTimeOutnotice()
    {
        $query = <<<SQL
        SELECT ID,
               DSNOME,
               QTDIAS,
               IDATIVO
          FROM INFORMATIVOECOMSILE
          WHERE CDPUBLICOALVO = 'OM'
SQL;
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query);
    }

    public function saveTimeOutNotice($connection, $id, $name, $qtDias, $cdoperador, $nrorg) {
        $query = <<<SQL
        INSERT INTO INFORMATIVOECOMSILE
               (ID, DSNOME, QTDIAS, IDATIVO, NRORG, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, CDPUBLICOALVO)
        VALUES (:ID, :DSNOME, :QTDIAS, 'S', :NRORG, SYSDATE, :NRORGINCLUSAO, :CDOPERINCLUSAO, 'OM')
SQL;
        $params = array(
            'ID' => $id,
            'DSNOME' => $name,
            'QTDIAS' => $qtDias,
            'NRORG' => $nrorg,
            'NRORGINCLUSAO' => $nrorg,
            'CDOPERINCLUSAO' => $cdoperador
        );

        $connection->executeStatement($query, $params);
    }

    public function updateTimeOutNotice($connection, $id, $name, $qtDias, $cdoperador, $nrorg) {
        $query = <<<SQL
        UPDATE INFORMATIVOECOMSILE
            SET DSNOME = :DSNOME,
                QTDIAS = :QTDIAS,
                IDATIVO = 'S',
                NRORG = :NRORG,
                DTULTATU = SYSDATE,
                NRORGULTATU = :NRORGULTATU,
                CDOPERULTATU = :CDOPERULTATU
        WHERE ID = :ID
SQL;
        $params = array(
            'ID' => $id,
            'DSNOME' => $name,
            'QTDIAS' => $qtDias,
            'NRORG' => $nrorg,
            'NRORGULTATU' => $nrorg,
            'CDOPERULTATU' => $cdoperador
        );

        $connection->executeStatement($query, $params);
    }

    public function deleteTimeOutNotice($connection, $id) {
        $query = <<<SQL
            DELETE FROM INFORMATIVOECOMSILE
            WHERE ID = :ID
SQL;
        $params = array(
            'ID' => $id
        );

        $connection->executeQuery($query, $params);
    }

    public function validaRegistroIgual($connection, $id, $name)
    {
        $query = <<<SQL
        SELECT COUNT(*) AS QTD
        FROM INFORMATIVOECOMSILE
        WHERE DSNOME = :DSNOME
        AND ID != :ID
        AND CDPUBLICOALVO = 'OM'
SQL;

        $params = array(
            'ID' => $id,
            'DSNOME' => $name
        );

        return $connection->executeQuery($query, $params)->fetchAllAssociative();
    }

    public function findProdutoNotice()
    {
        $query = <<<SQL
        SELECT DISTINCT OMP.ID AS NRPRODUTO, OMP.NMPRODUTO
        FROM OM_PRODUTO_ORG OPO
            INNER JOIN OM_PRODUTO OMP ON OPO.NRPRODUTO_ID = OMP.ID
            INNER JOIN ORGANIZACAO ORG ON ORG.NRORG = OPO.NRORG_ID
            INNER JOIN PRODUCT PRO ON PRO.ID = OMP.NREXTERNAL_PRODUCT_ID
            INNER JOIN PROJECT PRJ ON PRJ.ID = PRO.PROJECT_ID AND PRODUCT_TYPE = 'C'
        WHERE 1 = 1
        ORDER BY OMP.NMPRODUTO
SQL;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query);
    }

    public function insertProdutoNotice($cdinforproduto, $produto, $nrinfecommerce, $nrorg)
    {
            $query = <<<SQL
            INSERT
              INTO INFORMATIVO_PRODUTO
                   (CDINFORPRODUTO,
                   NRPRODUTO,
                   NRINFECOMMERCE,
                   DTINCLUSAO,
                   DTULTATU,
                   IDATIVO,
                   NRORG,
                   NRORGINCLUSAO,
                   CDOPERINCLUSAO)
            VALUES (:CDINFORPRODUTO,
                   :NRPRODUTO,
                   :NRINFECOMMERCE,
                   TRUNC(CAST(SYSDATE AS DATE)),
                   TRUNC(CAST(SYSDATE AS DATE)),
                   :IDATIVO,
                   :NRORG,
                   :NRORGINCLUSAO,
                   :CDOPERINCLUSAO)
SQL;
            $params = array(
                'CDINFORPRODUTO' => $cdinforproduto,
                'NRPRODUTO' => $produto,
                'NRINFECOMMERCE' => $nrinfecommerce,
                'IDATIVO'       => 'S',
                'NRORG'         => $nrorg,
                'NRORGINCLUSAO' => $nrorg,
                'CDOPERINCLUSAO' => $this->getCdoperador(),
            );

        $entityManager = InstanceProvider::getLibConnectionCustom()->updateVpdConnection($nrorg);
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        return $entityManager->getConnection()->executeQuery($query, $params);   
    }

    public function getProdutosByComunicadoId($comunicadoId)
    {
        $query = <<<SQL
        SELECT NRPRODUTO FROM INFORMATIVO_PRODUTO WHERE NRINFECOMMERCE = :comunicadoId
SQL;

        $params = array(
            "comunicadoId" => $comunicadoId
        );

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }

    public function deleteProdutosNotice($connection, $comunicadoId, $produto)
    {
        $query = <<<SQL
            DELETE FROM INFORMATIVO_PRODUTO 
            WHERE NRINFECOMMERCE = :comunicadoId 
            AND NRPRODUTO = :produto
SQL;
        $params = array(
            'comunicadoId' => $comunicadoId,
            'produto'        => $produto
        );

        $connection->executeQuery($query, $params);
    }

    public function deleteAllProdutosNotice($connection, $comunicadoId)
    {
        $query = <<<SQL
            DELETE FROM INFORMATIVO_PRODUTO 
            WHERE NRINFECOMMERCE = :comunicadoId
SQL;
        $params = array(
            'comunicadoId' => $comunicadoId
        );

        $connection->executeQuery($query, $params);
    }
}
