<?php


namespace OrganizationsManagement\Repository;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;
use OrganizationsManagement\Util\InstanceProvider;

class Operador extends RepositoryBase
{

    use TQuery;

    const ENTITY_NAME = 'Teknisa\Model\Geral\Entities\Operador';
    /**
     * @var EntityManager
     */
    private $entityManager;

    public function __construct()
    {
        parent::__construct(self::ENTITY_NAME);
        $this->entityManager = InstanceProvider::getEntityManager();
    }

    /**
     * @return array
     */
    public function findAllOperadores(): array
    {
        $sql = <<<SQL
               SELECT DISTINCT OP.CDOPERADOR, NMOPERADOR,
                      DECODE(IDOPERMASTERCP, 'N', 'Não', 'S', 'Sim') AS IDOPERMASTERCP,
                      DECODE(IDOPERMASTERZMART, 'N', 'Não', 'S', 'Sim') AS IDOPERMASTERZMART
                 FROM OPERADOR OP
                WHERE OP.IDOPERATIVO <> 'I'
             ORDER BY OP.NMOPERADOR
SQL;
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    /**
     * @param $cdCliente
     * @return array
     */
    public function findOperadoresPorCliente($cdCliente): array
    {
        $sql = <<<SQL
               SELECT OC.CDOPERADOR, OP.NMOPERADOR, OC.NRPRIOPERCLI,
                      OC.CDCLIENTE, OC.IDEMAILENCSO,
                      DECODE(OC.IDTPOPERCLIE, 'R', 'Responsável', 'C', 'Cliente') AS IDTPOPERCLIE
                 FROM OPERADOR OP, OPERCLIE OC
                WHERE OP.CDOPERADOR = OC.CDOPERADOR
                  AND OC.CDCLIENTE = :P_CDCLIENTE
             ORDER BY OP.NMOPERADOR
SQL;
        $param = array(
            'P_CDCLIENTE' => $cdCliente,
        );
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $param);
    }

    /**
     * @param $cdcliente
     * @return array
     */
    public function findRelacionarOperadoresComCliente($cdcliente): array
    {
        $sql = <<<SQL
               SELECT O.CDOPERADOR, O.NMOPERADOR
                 FROM OPERADOR O
                WHERE O.IDOPERATIVO = 'A'
                  AND O.CDOPERADOR NOT IN (SELECT OC.CDOPERADOR
                                             FROM OPERADOR OP, OPERCLIE OC
                                            WHERE OP.CDOPERADOR = OC.CDOPERADOR
                                              AND OC.CDCLIENTE = :P_CDCLIENTE)
            ORDER BY O.NMOPERADOR
SQL;
        $param = array(
            'P_CDCLIENTE' => $cdcliente,
        );
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $param);
    }

    public function getFindPadGer(): array
    {
        $sql = <<<SQL
        SELECT CDOPERADOR, NMOPERADOR
FROM OPERADOR
WHERE IDOPERATIVO = 'A'
SQL;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql); 
    }
}
