import { Metadata } from '@zeedhi/core';
import { Text, Button } from '@zeedhi/common';
import { TekGrid, TekRestDatasource, TekTreeGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class OrganizationReportController {
  public organizationReportGridIsVisible: boolean = true;

  get organizationReportDetailIsVisible() {
    return !this.organizationReportGridIsVisible;
  }

  public clickBackButtonOrganizationReport() {
    this.organizationReportGridIsVisible = true;
  }

  public organizationReportGridCurrentRow: any = {};

  public organizationReportDetailHeaderTitle = '';

  public cdPais: any;

  public siglaEstado: any;

  public estadoReadOnly: boolean = true;

  public municipioReadOnly: boolean = true;

  public redirecionamentoRC: string = '';

  public async openFilterModal({ component }: any) {
    const filterButton = Metadata.getInstance<Button>(`${component.name}_filterButton`);
    filterButton.click({} as Event, {} as HTMLElement);

    this.setAsActiveKey();
  }

  public onChangeCountry({ component }: any) {
    if (component.value !== null) {
      this.estadoReadOnly = false;
      this.cdPais = component.value;
    } else {
      this.estadoReadOnly = true;
      this.cdPais = null;
    }
  }

  public onClickEstado({ component }: any) {
    if (this.cdPais) {
      component.datasource.setFilter({
        CDPAIS: this.cdPais,
      });
    } else {
      component.value = null;
      component.datasource.data = [];
    }
  }

  public onChangeState({ component }: any) {
    this.siglaEstado = component.value;
    if (this.siglaEstado) {
      this.municipioReadOnly = false;
    } else {
      this.municipioReadOnly = true;
  }
}

  public onClickCity({ component }: any) {
    component.datasource.setFilter({
      CDPAIS: this.cdPais,
      SGESTADO: this.siglaEstado,
    });
  }

  public setAsActiveKey() {
    const organizationReportGrid = Metadata.getInstance<TekGrid>('organizationReportGrid');
    if (organizationReportGrid.datasource instanceof TekRestDatasource) {
      organizationReportGrid.datasource.dynamicFilter = {
        ATIVO: [
          {
            operation: 'CONTAINS',
            relation: 'AND',
            value: 'S',
          },
        ],
      };
    }
  }

  public async organizationReportRowClick({ row }: any) {
    const nmOrganizacao = Metadata.getInstance<Text>('NMORGANIZACAO');
    const nmRazaoSocial = Metadata.getInstance<Text>('NMRAZSOCIALORG');
    const asActive = Metadata.getInstance<Text>('ATIVO');
    const nmProdutos = Metadata.getInstance<Text>('NMPRODUTO');
    const dtInclusao = Metadata.getInstance<Text>('DTINCLUSAO');
    const dsUrlAcesso = Metadata.getInstance<Text>('DSURLACESSO');
    const dsUrlRedirect = Metadata.getInstance<Text>('DSURLREDIRECT');
    const dUrlCustom = Metadata.getInstance<Text>('DSURLCUSTOM');
    const nmPais = Metadata.getInstance<Text>('NMPAIS');
    const nmEstado = Metadata.getInstance<Text>('NMESTADO');
    const nmMunicipio = Metadata.getInstance<Text>('NMMUNICIPIO');
    this.organizationReportGridCurrentRow = row;
    this.organizationReportDetailHeaderTitle = row.NMORGANIZACAO;
    this.organizationReportGridIsVisible = false;
    nmOrganizacao.text = row.NMORGANIZACAO;
    nmRazaoSocial.text = row.NMRAZSOCIALORG;
    asActive.text = row.ATIVO;
    nmProdutos.text = row.NMPRODUTO;
    dtInclusao.text = row.DTINCLUSAO;
    dsUrlAcesso.text = row.DSURLACESSO;
    dsUrlRedirect.text = row.DSURLREDIRECT;
    if (row.IDUTREDIRECTRC === 'S') {
      this.redirecionamentoRC = 'Sim';
    } else if (row.IDUTREDIRECTRC === 'N') {
      this.redirecionamentoRC = 'Não';
    }
    dUrlCustom.text = row.DSURLCUSTOM;
    nmPais.text = row.NMPAIS;
    nmEstado.text = row.NMESTADO;
    nmMunicipio.text = row.NMMUNICIPIO;

    const dateParts = row.DTINCLUSAO.split(' ');
    dtInclusao.text = dateParts[0];

    const organizationReportTreeGrid = Metadata.getInstance<TekTreeGrid>('organizationReportTreeGrid');
    try {
      organizationReportTreeGrid.datasource.setFilter({
        PROJECT_ID: row.ID,
        PRODUCT_ORG: row.ORGID,
      });
    } catch (e) {
      console.error(e);
    }
    await this.orgStatusRow({ row });
    await this.hospedagemSaasRow({ row });
    await this.isPrimaryRow({ row });
    await this.asActiveRow({ row });
  }

  public async asActiveRow({ row }: any) {
    const ativo = Metadata.getInstance<Text>('ATIVO');
    ativo.text = row.ATIVO;

    const nameMapping: { [key: string]: string } = {
      S: 'Sim',
      N: 'Não',
    };
    if (typeof ativo.text === 'string' && nameMapping[ativo.text]) {
      ativo.text = nameMapping[ativo.text];
    }
  }

  public async orgStatusRow({ row }: any) {
    const orgStatus = Metadata.getInstance<Text>('ORGSTATUS');
    orgStatus.text = row.ORGSTATUS;

    const nameMapping: { [key: string]: string } = {
      P: 'Base Preparada',
      D: 'Base Bloqueada',
      Z: 'Base Preparada (Zmart Incluso)',
      C: 'Gerar Estrutura',
      Q: 'Gerar Dados Iniciais',
      S: 'Geração de Estrutura Solicitada',
      I: 'Organização Inativada',
      R: 'Organização Inativada - Base Removida',
      N: 'Preparação Pendente (Local/Hosting)',
      L: 'Organização Local/Hosting Preparada',
      Y: 'Organização Local/Hosting Preparada (Zmart Incluso)',
      J: 'Organização Local/Hosting Inativada',
      O: 'Preparação Pendente (Secundária)',
      M: 'Organização Secundária Preparada',
      X: 'Organização Secundária Preparada (Zmart Incluso)',
      K: 'Organização Secundária Inativada',
    };
    if (typeof orgStatus.text === 'string' && nameMapping[orgStatus.text]) {
      orgStatus.text = nameMapping[orgStatus.text];
    }
  }

  public async hospedagemSaasRow({ row }: any) {
    const hospedagem = Metadata.getInstance<Text>('IDORGSAAS');
    hospedagem.text = row.IDORGSAAS;

    const nameMapping: { [key: string]: string } = {
      S: 'SaaS',
      N: 'Local',
      H: 'Hosting',
    };
    if (typeof hospedagem.text === 'string' && nameMapping[hospedagem.text]) {
      hospedagem.text = nameMapping[hospedagem.text];
    }
  }

  public async isPrimaryRow({ row }: any) {
    const isPrimary = Metadata.getInstance<Text>('IS_PRIMARY');
    isPrimary.text = row.IS_PRIMARY;

    const nameMapping: { [key: string]: string } = {
      P: 'Primária',
      S: 'Secundária',
    };
    if (typeof isPrimary.text === 'string' && nameMapping[isPrimary.text]) {
      isPrimary.text = nameMapping[isPrimary.text];
    }
  }

  public getRowColor({ row }: any) {
    const organizationReportTreeGrid = Metadata.getInstance<TekTreeGrid>('organizationReportTreeGrid');
    const elements = organizationReportTreeGrid.datasource.data;
    Utilities.loadViewButtonCss(elements);
    const currentRow = elements.find((element) => element.ID === row.ID);
    if (currentRow) {
      if (currentRow.PARCIAL) {
        return 'yellow';
      }
      if (currentRow.ATIVO === 'S') {
        return 'blue';
      }
      if (currentRow.ATIVO === 'N') {
        return 'gray';
      }
    }
    return '';
  }
}
